#!/bin/bash
set -u

# Get EBS volumes with Name tag of <host>-root, <host>-hxdepots, and <host>-hxlogs,
# e.g. perforce-01-root, perforce-01-hxdepots, and perforce-01-hxlogs. Take snapshots,
# and tag them with the current journal counter.

function msg () { echo -e "$*"; }
function errmsg () { msg "\\nError: ${1:-Unknown Error}\\n"; ExitCode=1; }
function bail () { errmsg "${1:-Unknown Error}"; exit "${2:-1}"; }

declare ThisScript="${0##*/}"
declare Version="1.2.0"
declare ThisHost=$(hostname -s)
declare VolumeBaseName=
declare VolumeName=
declare SnapshotName=
declare VolumeId=
declare SnapshotId=
declare Cmd=
declare -i ExitCode=0
declare currentJournal

currentJournal=$(p4 counters | grep 'journal' | cut -d ' ' -f 3)

msg "Started ${0##*/} v$Version at $(date)."

for VolumeBaseName in root hxdepots hxlogs; do
   VolumeName="${ThisHost}-${VolumeBaseName}"
   VolumeId=$(aws ec2 describe-volumes --filters Name=tag:Name,Values=$VolumeName --query 'Volumes[*].{ID:VolumeId}' --output text)

   if [[ -z "$VolumeId" ]]; then
      errmsg "Could not determine VolumeId for $VolumeName. Skipping it."
      continue
   fi

   msg "Snapshotting volume $VolumeName [$VolumeId]."
   Cmd="aws ec2 create-snapshot --description Snapshot-$VolumeName --volume-id $VolumeId --query SnapshotId --output text"

   msg "Running: $Cmd"
   SnapshotId=$($Cmd)
   if [[ -n "$SnapshotId" ]]; then
      msg "Snapshot created for $VolumeName on $ThisHost with Id: $SnapshotId."
      SnapshotName="${VolumeName}-${currentJournal}"
      Cmd="aws ec2 create-tags --resources $SnapshotId --tags Key=Host,Value=$ThisHost Key=Contents,Value=CheckpointsAndArchives Key=Name,Value=$SnapshotName --output text"
      if $Cmd; then
         msg "Verified: Resource tags applied."
      else
         errmsg "Failed to apply tags to Snapshot $SnapshotId."
      fi
   else
      errmsg "Failed to create snapshot for $VolumeName."
   fi
done

if [[ "$ExitCode" -eq 0 ]]; then
   msg "Processing completed OK."
else
   msg "Processing completed WITH ERRORS. Review the output above."
fi

exit $ExitCode
